nelx=40;nely=30;volfrac=0.5;

hX=nelx*(repmat(1/4,1));% one hole
hY=nely*(kron(1,1/2));
% hX=nelx*(repmat([1/4,3/4],1,2)); % four holes
% hY=nely*(kron([1/4,3/4],ones(1,2)));

r = nely*0.08;%RADIUS OF INITIAL HOLES

[X,Y] = meshgrid(0:1:nelx,0:1:nely);
dX = bsxfun(@minus,repmat(X,[1,1,numel(hX)]),reshape(hX,1,1,numel(hX)));
dY = bsxfun(@minus,repmat(Y,[1,1,numel(hY)]),reshape(hY,1,1,numel(hY)));

Phi = max(-9,min(9,min(sqrt(dX.^2+dY.^2)-r,[],3))); 
 E0=1;Emin = 1e-3; 
%% RADIAL BASIS FUNCTION INITIALIZATION
cRBF = 1e-4;%RBF PARAMETERRBF
nNode = (nely+1)*(nelx+1);
Ax = bsxfun(@minus,X(:),X(:)');
Ay = bsxfun(@minus,Y(:),Y(:)');
A = sqrt(Ax.^2+Ay.^2+cRBF^2);
G = [A,ones(nNode,1),X(:),Y(:);[ones(1,nNode);X(:)';Y(:)'],zeros(3,3)];
pGpX = [Ax./A,repmat([0,1,0],nNode,1);repmat([0;1;0],1,nNode),zeros(3,3)];
pGpY = [Ay./A,repmat([0,0,1],nNode,1);repmat([0;0;1],1,nNode),zeros(3,3)];
Alpha = G\[Phi(:);0;0;0];
%% FINITE ELEMENT ANALYSIS PREPARATION

eleN1 = repmat((1:nely)',1,nelx)+kron(0:nelx-1,(nely+1)*ones(nely,1));
eleNode = repmat(eleN1(:),1,4)+repmat([0,nely+[1,2],1],nelx*nely,1);
edofMat = kron(eleNode,[5,5,5,5,5])+repmat([-4,-3,-2,-1,0],nelx*nely,4); 
iK = reshape(kron(edofMat,ones(20,1))',400*nelx*nely,1);
jK = reshape(kron(edofMat,ones(1,20))',400*nelx*nely,1); 
nodecord=zeros(nNode,5);

nodecord(:,1)=reshape(X,[],1);
nodecord(:,2)=reshape(Y,[],1);
%% BOUNDARY CONDITION DEFINITION 

fixednodes=[1:1:(nely+1),(nely+2):(nely+1):nNode,2*(nely+1):(nely+1):nNode,(nelx*(nely+1)+2):1:(nNode-1)];
fixeddofs=kron(fixednodes,[5,5,5,5,5])+repmat([-4,-3,-2,-1,0],1,length(fixednodes));  %guzhi
alldofs=1:5*(nely+1)*(nelx+1);
freedofs    = setdiff(alldofs,fixeddofs);

%% MATERIAL PROPERTIES
Eb=205e5;vb=0.3;h1b=0.1; Densb=7900e-8;     %base plate
E1=10e4;v=0.5;h1=0.3; Dens=14e-6; yita=0.15;   %viscoelastic layer
 E1=E1*(1+1i*yita);
hn=(Eb*h1b^2+E1*h1*(2*h1b+h1))/(2*(Eb*h1b+E1*h1))-h1b/2;  %neutral axis

%% FEM matrix

Q_al=[E1/(1-v^2) v*E1/(1-v^2) 0;v*E1/(1-v^2) E1/(1-v^2) 0;0 0 E1/(2*(1+v))];
      
        
 Me=ShellQuadElementMass(Dens,h1,nodecord(eleNode(1,1),:),nodecord(eleNode(1,2),:),...
        nodecord(eleNode(1,3),:),nodecord(eleNode(1,4),:));% mass matrix of viscoelastic layer

   
 Ke1=ShellQuadElementStiffVisco(Q_al,h1b,h1,hn,nodecord(eleNode(1,1),:),nodecord(eleNode(1,2),:),...
        nodecord(eleNode(1,3),:),nodecord(eleNode(1,4),:));% stiffness matrix of viscoelastic layer


Q_alb=[Eb/(1-vb^2) vb*Eb/(1-vb^2) 0;vb*Eb/(1-vb^2) Eb/(1-vb^2) 0;0 0 Eb/(2*(1+vb))];
       

     Meb=ShellQuadElementMass(Densb,h1b,nodecord(eleNode(1,1),:),nodecord(eleNode(1,2),:),...  % mass matrix of base layer
        nodecord(eleNode(1,3),:),nodecord(eleNode(1,4),:));%ShellQuadElementMass(NL,Dens,Z,ue1,ue2,ue3,ue4)
    Keb=ShellQuadElementStiff(Q_alb,h1b,nodecord(eleNode(1,1),:),nodecord(eleNode(1,2),:),...  
        nodecord(eleNode(1,3),:),nodecord(eleNode(1,4),:));% stiffness matrix of base layer without damping treatment
    Keb1=ShellQuadElementStiffBase(Q_alb,h1b,hn,nodecord(eleNode(1,1),:),nodecord(eleNode(1,2),:),...  
        nodecord(eleNode(1,3),:),nodecord(eleNode(1,4),:));%stiffness matrix of base layer with damping treatmentKe = ShellQuadElementStiff(NL,Q,Z,ue1,ue2,ue3,ue4)



Me=double(Me);
Meb=double(Meb);
Keb=double(Keb);

Ke1=double(Ke1);
Keb1=double(Keb1);

Kev=Ke1+Keb1;
Kevi=imag(Kev);
Kevr=real(Kev);
    
    
U1 = zeros(5*nNode,1);
U2 = zeros(5*nNode,1);
U3 = zeros(5*nNode,1);
U4 = zeros(5*nNode,1);
U5 = zeros(5*nNode,1);
U6 = zeros(5*nNode,1);

sMb = reshape(Meb(:)*ones(1,nelx*nely),400*nelx*nely,1);  
Mb = sparse(iK,jK,sMb); Mb = (Mb+Mb')/2;

%% ITERATION OPTIMIZATION 
nLoop = 200; nRelax = 30; 
dt = 0.5; delta = 10; mu = 30; gamma = 0.05;
Damp = zeros(nLoop,1); vol = zeros(nLoop,1);
fr1 = zeros(nLoop,1);
fr6 = zeros(nLoop,1);
for iT = 1:nLoop
%% FINITE ELEMENT ANALYSIS
[s,t] = meshgrid(-1:0.1:1,-1:0.1:1);
tmpPhi = (1-s(:)).*(1-t(:))/4*Phi(eleNode(:,1))'+(1+s(:)).*(1-t(:))/4*...
Phi(eleNode(:,2))'+(1+s(:)).*(1+t(:))/4*Phi(eleNode(:,3))'+...
(1-s(:)).*(1+t(:))/4*Phi(eleNode(:,4))'; 
eleVol = sum(tmpPhi>=0,1)'/numel(s);
vol(iT) = sum(eleVol)/(nelx*nely);


sM = reshape(Me(:)*((Emin+eleVol*(E0-Emin))'),400*nelx*nely,1);
M = sparse(iK,jK,sM); M = (M+M')/2;


sKb = reshape(Keb(:)*(ones(1,nelx*nely)-(Emin+eleVol*(E0-Emin))'),400*nelx*nely,1);
Kb = sparse(iK,jK,sKb); Kb = (Kb+Kb')/2;

sK = reshape(Kevr(:)*((Emin+eleVol*(E0-Emin))'),400*nelx*nely,1);
K = sparse(iK,jK,sK); K = (K+K')/2;


KT=Kb+K;MT=Mb+M;

[yy,lamt]=eigs(KT(freedofs,freedofs), MT(freedofs,freedofs),6,'SM');

fr=sqrt(lamt)/2/pi;
fr1(iT)=fr(6,6);
fr6(iT)=fr(1,1);

U1(freedofs,1) = yy(:,6);
U2(freedofs,1) = yy(:,5);
U3(freedofs,1) = yy(:,4);
U4(freedofs,1) = yy(:,3);
U5(freedofs,1) = yy(:,2);
U6(freedofs,1) = yy(:,1);


eleDamp1 = (sum((U1(edofMat)*(Kevi)).*U1(edofMat),2).*(eleVol))./(sum(sum((U1(edofMat)*Keb).*U1(edofMat),2).*(ones(nelx*nely,1)-eleVol)+sum((U1(edofMat)*Kevr).*U1(edofMat),2).*(Emin+eleVol*(E0-Emin))));
eleDamp2 = (sum((U2(edofMat)*(Kevi)).*U2(edofMat),2).*(eleVol))./(sum(sum((U2(edofMat)*Keb).*U2(edofMat),2).*(ones(nelx*nely,1)-eleVol)+sum((U2(edofMat)*Kevr).*U2(edofMat),2).*(Emin+eleVol*(E0-Emin))));
eleDamp3 = (sum((U3(edofMat)*(Kevi)).*U3(edofMat),2).*(eleVol))./(sum(sum((U3(edofMat)*Keb).*U3(edofMat),2).*(ones(nelx*nely,1)-eleVol)+sum((U3(edofMat)*Kevr).*U3(edofMat),2).*(Emin+eleVol*(E0-Emin))));
eleDamp4 = (sum((U4(edofMat)*(Kevi)).*U4(edofMat),2).*(eleVol))./(sum(sum((U4(edofMat)*Keb).*U4(edofMat),2).*(ones(nelx*nely,1)-eleVol)+sum((U4(edofMat)*Kevr).*U4(edofMat),2).*(Emin+eleVol*(E0-Emin))));
eleDamp5 = (sum((U5(edofMat)*(Kevi)).*U5(edofMat),2).*(eleVol))./(sum(sum((U5(edofMat)*Keb).*U5(edofMat),2).*(ones(nelx*nely,1)-eleVol)+sum((U5(edofMat)*Kevr).*U5(edofMat),2).*(Emin+eleVol*(E0-Emin))));
eleDamp6 = (sum((U6(edofMat)*(Kevi)).*U6(edofMat),2).*(eleVol))./(sum(sum((U6(edofMat)*Keb).*U6(edofMat),2).*(ones(nelx*nely,1)-eleVol)+sum((U6(edofMat)*Kevr).*U6(edofMat),2).*(Emin+eleVol*(E0-Emin))));



% eleDamp = 1/2*eleDamp1+1/2*eleDamp6;

 eleDamp = eleDamp5;
% eleDamp = eleDamp1;

Damp(iT) = sum(eleDamp);
%% DISPLAY RESULTS
fprintf('No.%i, Obj:%f, Vol:%f\n',[iT,Damp(iT),vol(iT)]);
figure(1); 
axes1 = axes('YTick',zeros(1,0),'XTickLabel',{},...
    'XTick',zeros(1,0),...
    'PlotBoxAspectRatio',[20 15 1],...
    'Layer','top',...
    'DataAspectRatio',[1 1 1]);
xlim(axes1,[1 41]);
ylim(axes1,[1 31]);
box(axes1,'on');
hold(axes1,'all');
contourf(Phi,[0,0]);
colormap([0.5,0.5,0.5]); set(gcf,'color','w'); 
saveas(gcf, 'figure(1)', 'jpg')

figure(2); surf(Phi); caxis([-12,12]); 
axis equal; axis([0,nelx,0,nely,-12,12]); view(3);
figure(3); subplot(2,1,1); plot(Damp(1:iT),'-'); title('Modal loss factor');
subplot(2,1,2); plot(vol(1:iT),'-'); title('Volume fraction');
% figure(4);subplot(2,1,1); plot(fr1(1:iT),'-'); title('1st modal frequency');
% subplot(2,1,2); plot(fr6(1:iT),'-'); title('6th modal frequency');


%% CONVERGENCE CHECK
if iT>nRelax && abs(vol(iT)-volfrac)/volfrac<1e-3 && ...
 all(abs(Damp(iT)-Damp(iT-9:iT-1))/Damp(iT)<1e-3)
break;
end
%% LAGRANGE MULTIPLIER
if iT<=nRelax
lag = mu*(vol(iT)-vol(1)+(vol(1)-volfrac)*iT/nRelax);
else
lag = lag+gamma*(vol(iT)-volfrac);
gamma = min(gamma+0.05,5);
end
%% LEVEL SET FUNCTION EVOLUTION 
gradPhi = sqrt((pGpX*Alpha).^2+(pGpY*Alpha).^2); 
indexDelta = (abs(Phi(:))<=delta);
DeltaPhi = zeros(size(Phi));
AC=0.6; 
DeltaPhi(indexDelta) = AC/delta*(1-Phi(indexDelta).^2/delta^2); 
eleDamp = reshape(eleDamp,nely,nelx);
eleDampLR = [eleDamp(:,1),eleDamp]+[eleDamp,eleDamp(:,end)];
nodeDamp = ([eleDampLR;eleDampLR(end,:)]+[eleDampLR(1,:);eleDampLR])/4;

B = (nodeDamp(:)/median(nodeDamp(:))-lag).*DeltaPhi(:)*delta;
Alpha = Alpha+dt*(G\[B;0;0;0]);
Alpha = Alpha/mean(gradPhi(unique(eleNode((eleVol<1 & eleVol>0),:))));
Phi = reshape(G(1:end-3,:)*Alpha,nely+1,nelx+1);
end